use std::time::{Duration, Instant};
use skunk2d::Vec2;

pub struct Interpolator {
    start: Instant,
    end: Instant,
    length: Duration
}

impl Interpolator {
    pub fn new(start: Instant, length: Duration) -> Self {
        Interpolator {
            start,
            end: start + length,
            length
        }
    }

    //interpolator that will always return 1.0f64
    pub fn fake() -> Self {
        //ugly <_<
        let meh = Instant::now() - Duration::from_secs(100);
        Interpolator::new(meh, Duration::from_secs(1))
    }

    pub fn get(&self, pos: Instant) -> f64 {
        (pos - self.start).div_duration_f64(self.length).max(0f64).min(1f64)
    }

    pub fn apply_vec2(&self, pos: Instant, v1: Vec2<i32>, v2: Vec2<i32>) -> Vec2<i32> {
        let p = self.get(pos);
        let delta = v2 - v1;
        if delta.x.abs() > 100 || delta.y.abs() > 100 {
            v2
        } else {
            Vec2 {
                x: (v1.x as f64 + delta.x as f64 * p).round() as i32,
                y: (v1.y as f64 + delta.y as f64 * p).round() as i32
            }
        }
    }
}
