use std::rc::Rc;
use std::time::Instant;
use skunk2d::{HexMap, Image, Vec2, Direction};
use crate::interpolator::Interpolator;
use crate::unit_kind::UnitKind;

pub struct Unit {
    pub kind: Rc<UnitKind>,
    pub pos: Vec2<i32>,
    pub interpolator: Interpolator,
    pub old_pos: Vec2<i32>,
    pub cooldown: i32
}

impl Unit {
    pub fn cool(&mut self) {
        self.cooldown = (self.cooldown - 1).max(0)
    }

    pub fn draw(&self, target: &mut Image, map: &HexMap, time: Instant) {
        let old_pos = map.coord_to_pixel(self.old_pos);
        let new_pos = map.coord_to_pixel(self.pos);
        let pos = self.interpolator.apply_vec2(time, old_pos, new_pos);
        //todo: don't hardcode offset :P
        target.draw_image( pos + Vec2{x: 0, y: -24}, &self.kind.sprite);
    }

    pub fn move_unit(&mut self, dir: Direction, map: &HexMap) {
        if self.cooldown == 0 {
            let new_pos = HexMap::get_neighbour(self.pos, dir);
            if map.is_valid_coord(new_pos) {
                self.set_pos(new_pos);
                self.cooldown = self.kind.speed;
            }
        }
    }

    pub fn set_pos(&mut self, pos: Vec2<i32>) {
        self.interpolator = Interpolator::new(Instant::now(), self.kind.movement_duration);
        self.old_pos = self.pos;
        self.pos = pos;
    }

    pub fn add_pos(&mut self, pos: Vec2<i32>) {
        self.set_pos(self.pos + pos);
    }
}